/*--------------------------------------------------------------*/
/* FILE: output.pl */

% Code is displayed on current output

disp_code(Code):-
	disp_code(Code, 1, _).


% disp_code(Code, N0, N): Code is displayed so that its first
% instruction is at location N0, and the first unused location is N.

disp_code(instr(Mnem, Addr), N, N1):-
	disp_line(N, Mnem, Addr),
	N1 is N+1.
disp_code(label(_), N, N).
disp_code(block(L), N, M):-
	L>0, disp_line(N, 'DATA', ''),
	L1 is L-1, N1 is N+1, disp_code(block(L1), N1, M).
disp_code(block(0), N, N).
disp_code((Code1;Code2), N0, N):-
	disp_code(Code1, N0, N1),
	disp_code(Code2, N1, N).


% An instruction consisting of a Mnem and Address at location N is
% displayed.

disp_line(N, Mnem, Addr):-
	write(N), write(:), fill(N, 5), write(Mnem), fill(Mnem, 8),
	write(Addr), nl.


% Spaces are output so that length of A + number of spaces = W.

fill(A, W):-
	(   atom(A) -> atom_length(A, N)
	;   number_codes(A, L), length(L, N)
	),
	Spaces is W-N,
	tab(Spaces).

tab(S) :-
	S > 0, !,
	write(' '), S1 is S-1,
	tab(S1).
tab(_).


