read_program(Prog):-
	read_tokens(TokenList),
	program(Prog,TokenList,[]).

program(P) --> 
	statements(P), 
	program_end.

program_end -->	
	[.]  ;  [end_of_file].	

statements(P) --> 
	statement(S0), rest_statements(S0,P).

rest_statements(S0,(S0;S)) -->
	[';'],statements(S).
rest_statements(S,S) --> {true}.

statement(assign(V,Expr)) -->
	name(V),[:=],expr(Expr).
statement(if(Test,Then,Else)) -->
	[if], test(Test),
	[then],	statement(Then),
	[else], statement(Else).
statement(while(Test,Do)) -->
	[while], test(Test),
	[do], statement(Do).
statement(read(V)) -->
	[read],name(V).
statement(write(Expr)) -->
	[write],expr(Expr).
statement(S) -->
	['('],statements(S),[')'].

test(test(Op,X1,X2)) -->
	expr(X1),comparison_op(Op),expr(X2).

expr(X) -->
	subexpr(2,X).

subexpr(N,X) -->
	{N>0,N1 is N-1},
	subexpr(N1,X1),
	restexpr(N,X1,X).
subexpr(0,X) -->
	name(X).
subexpr(0,X) -->
	number(X).
subexpr(0,X) -->
	['('], subexpr(2,X),[')'].

restexpr(N,X1,X) -->
	arith_op(N,Op),
	{N1 is N-1}, subexpr(N1,X2),
	restexpr(N,expr(Op,X1,X2),X).
restexpr(_,X,X) --> {true}.

comparison_op(Op) --> [Op], {comparison_op(Op)}.

comparison_op(=).
comparison_op(<).
comparison_op(>).
comparison_op(\=).
comparison_op(=<).
comparison_op(>=).

arith_op(1,*) --> [*].
arith_op(1,/) --> [/].
arith_op(2,+) --> [+].
arith_op(2,-) --> [-].

name(name(V)) -->
	[V], {atom(V)}.

number(const(N)) -->
	[N], {integer(N)}.
