/*--------------------------------------------------------------*/
/* FILE: parse_op.pl */

:-op(990, xfx, :=).
:-op(995, fy, if).
:-op(995, xfy, then).
:-op(995, xfy, else).
:-op(995, fy, while).
:-op(995, xfy, do).
:-op(995, fx, read).
:-op(995, fx, write).


% parse_statement(St, ASt): ASt is the abstract form of statement St.

parse_statement((S1;S2), (P1;P2)):-
	parse_statement(S1, P1),
	parse_statement(S2, P2).
parse_statement((V:=E), assign(name(V), Expr)):-
	parse_expr(E, Expr).
parse_statement((if C then S1 else S2), if(Test, Then, Else)):-
	parse_expr(C, Test),
	parse_statement(S1, Then),
	parse_statement(S2, Else).
parse_statement((while C do S), while(Test, Do)):-
	parse_expr(C, Test),
	parse_statement(S, Do).
parse_statement((read V), read(name(V))).
parse_statement((write E), write(Expr)):-
	parse_expr(E, Expr).


% parse_expr(Expr, AExpr): AExpr is the abstract form of expression
% Expr.

parse_expr(C, const(C)):-integer(C).
parse_expr(Name, name(Name)):-atom(Name).
parse_expr(E, test(Op, Expr1, Expr2)):-
	E =.. [Op, E1, E2],
	testop(Op), !,
	parse_expr(E1, Expr1),
	parse_expr(E2, Expr2).
parse_expr(E, expr(Op, Expr1, Expr2)):-
	E =.. [Op, E1, E2],
	parse_expr(E1, Expr1),
	parse_expr(E2, Expr2).


% testop(Op): Op is a test operator.

testop(<).
testop(=<).
testop(\=).
testop(>).
testop(>=).
testop(=).


% A program is read and its abstract tree form returned in Prog.

read_program(Prog):-
	read(Term),
	parse_statement(Term, Prog).
